@@ -19,6 +19,7 @@
 
 #include <stddef.h>
 #include <stdlib.h>
+#include <ctype.h>
 #include "gpxview.h"
 
 #define GCONF_PATH         "/apps/gpxview/"
@@ -30,6 +31,8 @@
 #define GCONF_KEY_LOC_LON  GCONF_PATH "location%d/longitude"
 #define GCONF_KEY_LOC_CNT  GCONF_PATH "location_entries"
 
+#define GCONF_KEY_CLOSED   GCONF_PATH "closed/%s"
+
 #include <string.h>
 
 enum {
@@ -85,6 +88,42 @@ static store_t store[] = {
   { NULL, -1, -1 }
 };
 
+static char *get_basename(char *name) {
+  char *p = strrchr(name, '/');
+  if(!p) p = name;
+  else   p = p+1;
+
+  g_assert(*p);
+
+  /* escape all non alnum characters */
+  p = g_strdup(p);
+  int i;
+  for(i=0;i<strlen(p);i++)
+    if(!isalnum(p[i]))
+      p[i] = '_';
+
+  return p;
+}
+
+void gconf_remove_closed_name(appdata_t *appdata, char *filename) {
+  char *key = g_strdup_printf(GCONF_KEY_CLOSED, get_basename(filename));
+  gconf_client_unset(appdata->gconf_client, key, NULL);
+  g_free(key);
+}
+
+void gconf_save_closed_name(appdata_t *appdata, char *filename, char *name) {
+  char *key = g_strdup_printf(GCONF_KEY_CLOSED, get_basename(filename));
+  gconf_client_set_string(appdata->gconf_client, key, name, NULL);
+  g_free(key);
+}
+
+char *gconf_restore_closed_name(appdata_t *appdata, char *filename) {
+  char *key = g_strdup_printf(GCONF_KEY_CLOSED, get_basename(filename));
+  char *ret = gconf_client_get_string(appdata->gconf_client, key, NULL);
+  g_free(key);
+  return ret;
+}
+
 void gconf_save_state(appdata_t *appdata) {
   int entries = 0;
 
@@ -169,12 +208,22 @@ void gconf_load_state(appdata_t *appdata) {
     char *fname = gconf_client_get_string(appdata->gconf_client, str, NULL);
 
     if(fname) {
-      if(g_file_test(fname, G_FILE_TEST_IS_DIR)) 
-	*gpx = gpx_parse_dir(dialog, fname);
-      else
-	*gpx = gpx_parse(dialog, fname);
-      
-      free(fname);
+      /* check if there's a valid name stored for this file. */
+      /* if yes it's a "closed" file */
+      char *name = gconf_restore_closed_name(appdata, fname);
+      if(name) {
+	*gpx = g_new0(gpx_t, 1);
+	(*gpx)->filename = fname;
+	(*gpx)->name = g_strdup(name);
+	(*gpx)->closed = TRUE;
+      } else {
+	if(g_file_test(fname, G_FILE_TEST_IS_DIR)) 
+	  *gpx = gpx_parse_dir(dialog, fname);
+	else
+	  *gpx = gpx_parse(dialog, fname);
+
+	free(fname);
+      }
     }
 
     /* use next gpx entry of this was loaded successfully */
