@@ -25,7 +25,7 @@
 
 
 #ifdef USE_MAEMO
-#ifdef USE_GPSBT
+#ifdef ENABLE_GPSBT
 #include <gpsbt.h>
 #include <gpsmgr.h>
 #endif
