@@ -112,7 +112,7 @@ typedef struct {
   struct html_view *html_view;
 
   /* this is being activated and de-activated on selection/deselection */
-  GtkWidget *menu_remove;
+  GtkWidget *menu_remove, *menu_close;
 
   /* do enable/disable the edit entries */
   GtkWidget *menu_cut, *menu_copy, *menu_paste;
@@ -158,7 +158,9 @@ typedef struct {
 
   /* reference to change this to reflect fullscreen by hard button */
   /* and the ability to zoom */
+#if MAEMO_VERSION_MAJOR < 5
   GtkWidget *fullscreen_menu_item;
+#endif
   GtkWidget *zoomin_menu_item, *zoomout_menu_item;
 
   /* keep track of current "path" in the bread crumb trail */
