@@ -27,6 +27,8 @@
 #include "custom_rating_renderer.h"
 #include "custom_type_renderer.h"
 
+#include <hildon/hildon-banner.h>
+
 #include <locale.h>
 
 extern char *strcasestr (__const char *__haystack, __const char *__needle);
@@ -96,7 +98,7 @@ void errorf(const char *fmt, ...) {
 
   gtk_window_set_title(GTK_WINDOW(dialog), _("ERROR"));
 
- gtk_dialog_run(GTK_DIALOG(dialog));
+  gtk_dialog_run(GTK_DIALOG(dialog));
   gtk_widget_destroy(dialog);
 
   free(buf);
@@ -405,6 +407,13 @@ static GtkWidget *cachelist_create(appdata_t *appdata, gpx_t *gpx,
 			     G_TYPE_BOOLEAN);
 
   GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
+  
+#ifdef USE_MAEMO
+  /* this is rather slow under maemo, so draw some busy indicator */
+  GtkWidget *animation = 
+    hildon_banner_show_animation(GTK_WIDGET(appdata->window), 
+				 NULL, _("Updating...")); 
+#endif
 
   GtkTreeIter sel_iter;
   gboolean sel_iter_valid = FALSE;
@@ -455,9 +464,20 @@ static GtkWidget *cachelist_create(appdata_t *appdata, gpx_t *gpx,
       }
     }
 
+#ifdef USE_MAEMO
+    /* give gtk a chance to update the busy indicator */
+  while(gtk_events_pending())
+    gtk_main_iteration();
+#endif
+
     cache = cache->next;
   }
 
+#ifdef USE_MAEMO
+  /* close the busy indicator */
+  gtk_widget_destroy(animation);
+#endif
+
   gtk_tree_view_set_model(GTK_TREE_VIEW(view), GTK_TREE_MODEL(store));
   g_object_unref(store);
 
@@ -534,11 +554,50 @@ enum {
   GPXLIST_COL_NAME,
   GPXLIST_COL_DATE,
   GPXLIST_COL_CACHES,
+  GPXLIST_COL_OPEN,
   GPXLIST_COL_DATA,
   GPXLIST_NUM_COLS
 } ;
 
-void gpxlist_view_onRowActivated(GtkTreeView *treeview,
+static GdkPixbuf *gpx_icon_get(gpx_t *gpx) {
+  if(gpx->filename && g_file_test(gpx->filename, G_FILE_TEST_IS_DIR)) 
+    return icon_get(ICON_FILE, 1);
+
+  if(gpx->filename&& !strcasecmp(gpx->filename+strlen(gpx->filename)-4,".zip")) 
+    return icon_get(ICON_FILE, 2);
+
+  return icon_get(ICON_FILE, 0);
+}
+
+static void gpxlist_set(GtkListStore *store, GtkTreeIter *iter, gpx_t *gpx) {
+  char date_str[32], cnum[32];
+
+  if(gpx->year && gpx->month && gpx->day) {
+    GDate *date = g_date_new_dmy(gpx->day, gpx->month, gpx->year);
+    g_date_strftime(date_str, sizeof(date_str), "%x", date);
+    g_date_free(date);
+  } else
+    strcpy(date_str, "---");
+
+  char *fname = strrchr(gpx->filename, '/');
+  if(!fname) fname = gpx->filename;
+  else       fname++;  /* skip '/' */
+  
+  snprintf(cnum, sizeof(cnum), "%d", gpx_total_caches(gpx));
+  
+  /* Append a row and fill in some data */
+  gtk_list_store_set(store, iter,
+		     GPXLIST_COL_ICON, gpx_icon_get(gpx),
+		     GPXLIST_COL_FILENAME, fname,
+		     GPXLIST_COL_NAME, gpx->name,
+		     GPXLIST_COL_DATE, gpx->closed?NULL:date_str,
+		     GPXLIST_COL_OPEN, !gpx->closed,
+		     GPXLIST_COL_CACHES, gpx->closed?NULL:cnum,
+		     GPXLIST_COL_DATA, gpx,
+		     -1);
+}
+
+static void gpxlist_view_onRowActivated(GtkTreeView *treeview,
 			  GtkTreePath        *path,
 			  GtkTreeViewColumn  *col,
 			  gpointer            userdata) {
@@ -549,6 +608,50 @@ void gpxlist_view_onRowActivated(GtkTreeView *treeview,
   if (gtk_tree_model_get_iter(model, &iter, path)) {
     gpx_t *gpx;
     gtk_tree_model_get(model, &iter, GPXLIST_COL_DATA, &gpx, -1);
+
+    /* this gpx file may be closed. Since the user definitely wants */
+    /* to use it, we just open it again */
+    if(gpx->closed) {
+      gpx_dialog_t *dialog = gpx_busy_dialog_new(GTK_WIDGET(appdata->window));
+      gpx_t *new = NULL;
+    
+      if(g_file_test(gpx->filename, G_FILE_TEST_IS_DIR)) 
+	new = gpx_parse_dir(dialog, gpx->filename);
+      else
+	new = gpx_parse(dialog, gpx->filename);
+
+      if(new) {
+	gpx_t **prev = &(appdata->gpx);
+	while(*prev && *prev != gpx)
+	  prev = &(*prev)->next;
+
+	/* this entry _must_ be in the list */
+	g_assert(*prev);
+
+	/* replace gpx entry with the new "open" one */
+	(*prev) = new;
+	new->next = gpx->next;
+	gpx->next = NULL;
+	
+	/* free old closed one */
+	gpx_free(gpx);
+
+	gpx = new;
+
+	/* finally update the visible list */
+	gpxlist_set(appdata->gpxstore, &iter, gpx);
+
+	/* and remove gconf entry */
+	gconf_remove_closed_name(appdata, gpx->filename);
+
+	gtk_widget_set_sensitive(appdata->menu_close, TRUE);
+      } else {
+	printf("unable to reopen file %s\n", gpx->filename);
+	return;
+      }
+      
+      gpx_busy_dialog_destroy(dialog);
+    }
 #ifndef USE_MAEMO
     cachelist_dialog(appdata, gpx);
 #else
@@ -569,23 +672,19 @@ view_selection_func(GtkTreeSelection *selection, GtkTreeModel *model,
   appdata_t *appdata = (appdata_t*)userdata;
   GtkTreeIter iter;
     
-  if(gtk_tree_model_get_iter(model, &iter, path)) 
+  if(gtk_tree_model_get_iter(model, &iter, path)) {
+    gpx_t *gpx;
+    gtk_tree_model_get(model, &iter, GPXLIST_COL_DATA, &gpx, -1);
+
     gtk_widget_set_sensitive(appdata->menu_remove, !path_currently_selected);
+
+    if(!gpx->closed)
+      gtk_widget_set_sensitive(appdata->menu_close,  !path_currently_selected);
+  }
   
   return TRUE; /* allow selection state to change */
 }
 
-GdkPixbuf *gpx_icon_get(gpx_t *gpx) {
-  if(gpx->filename && g_file_test(gpx->filename, G_FILE_TEST_IS_DIR)) 
-    return icon_get(ICON_FILE, 1);
-
-  if(gpx->filename&& !strcasecmp(gpx->filename+strlen(gpx->filename)-4,".zip")) 
-    return icon_get(ICON_FILE, 2);
-
-  return icon_get(ICON_FILE, 0);
-}
-
-
 static GtkWidget *gpxlist_create_view_and_model(appdata_t *appdata, 
 						gpx_t *sel_gpx) {
   gpx_t               *gpx = appdata->gpx;
@@ -597,6 +696,7 @@ static GtkWidget *gpxlist_create_view_and_model(appdata_t *appdata,
 
   /* nothing selected yet */
   gtk_widget_set_sensitive(appdata->menu_remove,  FALSE);
+  gtk_widget_set_sensitive(appdata->menu_close,   FALSE);
 
   appdata->gpxview = gtk_tree_view_new ();
   GtkTreeSelection *selection = 
@@ -607,14 +707,20 @@ static GtkWidget *gpxlist_create_view_and_model(appdata_t *appdata,
   /* --- "Icon" column --- */
   renderer = gtk_cell_renderer_pixbuf_new();
   gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(appdata->gpxview),
-	      -1, "Icon", renderer, "pixbuf", GPXLIST_COL_ICON, NULL);
+		  -1, "Icon", renderer, 
+		  "pixbuf", GPXLIST_COL_ICON,
+	      //	          "sensitive", GPXLIST_COL_OPEN, 
+	          NULL);
 
   /* --- "FileName" column --- */
   if(appdata->gpxlist_items & GPXLIST_ITEM_FILENAME) {
     renderer = gtk_cell_renderer_text_new();
     gtk_tree_view_insert_column_with_attributes(
 		GTK_TREE_VIEW(appdata->gpxview),
-		-1, "Filename", renderer, "text", GPXLIST_COL_FILENAME, NULL);
+		-1, "Filename", renderer, 
+		"text", GPXLIST_COL_FILENAME, 
+		"sensitive", GPXLIST_COL_OPEN, 
+		NULL);
   }
 
   /* --- "Name" column --- */
@@ -622,7 +728,10 @@ static GtkWidget *gpxlist_create_view_and_model(appdata_t *appdata,
   g_object_set(renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL );
 
   GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes(
-		 "Name", renderer, "text", GPXLIST_COL_NAME, NULL);
+		 "Name", renderer, 
+		 "text", GPXLIST_COL_NAME,
+		 "sensitive", GPXLIST_COL_OPEN, 
+		 NULL);
   gtk_tree_view_column_set_expand(column, TRUE);
   gtk_tree_view_insert_column(GTK_TREE_VIEW(appdata->gpxview), column, -1);
 
@@ -632,7 +741,10 @@ static GtkWidget *gpxlist_create_view_and_model(appdata_t *appdata,
     g_object_set(renderer, "xalign", 1.0, NULL );
     gtk_tree_view_insert_column_with_attributes(
 		GTK_TREE_VIEW(appdata->gpxview),
-		-1, "Date", renderer, "text", GPXLIST_COL_DATE, NULL);
+		-1, "Date", renderer, 
+		"text", GPXLIST_COL_DATE,
+		"sensitive", GPXLIST_COL_OPEN, 
+		NULL);
   }
 
   /* --- "Number of caches" column --- */
@@ -641,42 +753,24 @@ static GtkWidget *gpxlist_create_view_and_model(appdata_t *appdata,
     g_object_set(renderer, "xalign", 1.0, NULL );
     gtk_tree_view_insert_column_with_attributes(
 		GTK_TREE_VIEW(appdata->gpxview),
-		-1, "#Caches", renderer, "text", GPXLIST_COL_CACHES, NULL);
+		-1, "#Caches", renderer, 
+		"text", GPXLIST_COL_CACHES,
+ 		"sensitive", GPXLIST_COL_OPEN, 
+		NULL);
   }
 
   /* build and fill the store */
   appdata->gpxstore = gtk_list_store_new(GPXLIST_NUM_COLS, GDK_TYPE_PIXBUF,
 			 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, 
-			 G_TYPE_INT, G_TYPE_POINTER);
+			 G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_POINTER);
 
   GtkTreePath *path = NULL; 
   GtkTreeIter sel_iter;
   gboolean sel_iter_valid = FALSE;
   while(gpx) {
     GtkTreeIter         iter;
-    char date_str[32];
-
-    if(gpx->year && gpx->month && gpx->day) {
-      GDate *date = g_date_new_dmy(gpx->day, gpx->month, gpx->year);
-      g_date_strftime(date_str, sizeof(date_str), "%x", date);
-      g_date_free(date);
-    } else
-      strcpy(date_str, "---");
-
-    char *fname = strrchr(gpx->filename, '/');
-    if(!fname) fname = gpx->filename;
-    else       fname++;  /* skip '/' */
-
-    /* Append a row and fill in some data */
     gtk_list_store_append(appdata->gpxstore, &iter);
-    gtk_list_store_set(appdata->gpxstore, &iter,
-			GPXLIST_COL_ICON, gpx_icon_get(gpx),
-			GPXLIST_COL_FILENAME, fname,
-			GPXLIST_COL_NAME, gpx->name,
-			GPXLIST_COL_DATE, date_str,
-			GPXLIST_COL_CACHES, gpx_total_caches(gpx),
-			GPXLIST_COL_DATA, gpx,
-			-1);
+    gpxlist_set(appdata->gpxstore, &iter, gpx);
 
     if(gpx == sel_gpx) {
       sel_iter = iter;
@@ -716,28 +810,9 @@ static GtkWidget *gpxlist_create_view_and_model(appdata_t *appdata,
 /* add last entry in gpx list to visual representation */
 static void gpxlist_add(appdata_t *appdata, gpx_t *new) {
   GtkTreeIter         iter;
-  char date_str[32];
 
-  if(new->year && new->month && new->day) {
-    GDate *date = g_date_new_dmy(new->day, new->month, new->year);
-    g_date_strftime(date_str, sizeof(date_str), "%x", date);
-  } else
-    strcpy(date_str, "---");
-
-  char *fname = strrchr(new->filename, '/');
-  if(!fname) fname = new->filename;
-  else       fname++;  /* skip '/' */
-  
-  /* Append a row and fill in some data */
   gtk_list_store_append(appdata->gpxstore, &iter);
-  gtk_list_store_set(appdata->gpxstore, &iter,
-		     GPXLIST_COL_ICON, gpx_icon_get(new),
-		     GPXLIST_COL_FILENAME, fname,
-		     GPXLIST_COL_NAME, new->name,
-		     GPXLIST_COL_DATE, date_str,
-		     GPXLIST_COL_CACHES, gpx_total_caches(new),
-		     GPXLIST_COL_DATA, new,
-		     -1);
+  gpxlist_set(appdata->gpxstore, &iter, new);
 
   /* and attach entry to end of list */
   gpx_t **gpx = &appdata->gpx;
@@ -811,6 +886,37 @@ cb_menu_precpos(GtkWidget *window, gpointer data) {
   precise_position((appdata_t *)data);
 }
 
+static void 
+cb_menu_close(GtkWidget *window, gpointer data) {
+  appdata_t *appdata = (appdata_t *)data;
+  GtkTreeSelection *selection;
+  printf("selected close\n");
+
+  /* the entry cannot be closed again */
+  gtk_widget_set_sensitive(appdata->menu_close,   FALSE);
+
+  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(appdata->gpxview));
+  GtkTreeModel     *model;
+  GtkTreeIter       iter;
+
+  if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
+    gpx_t *gpx;
+    gtk_tree_model_get(model, &iter, GPXLIST_COL_DATA, &gpx, -1);
+
+    g_assert(!gpx->closed);
+    gpx->closed = TRUE;
+
+    /* free all associated caches */
+    gpx_free_caches(gpx);
+
+    /* update entry */
+    gpxlist_set(appdata->gpxstore, &iter, gpx);
+
+    /* save name in gconf so we know this has been closed */
+    gconf_save_closed_name(appdata, gpx->filename, gpx->name);
+  }
+}
+
 static void 
 cb_menu_remove(GtkWidget *window, gpointer data) {
   appdata_t *appdata = (appdata_t *)data;
@@ -820,6 +926,7 @@ cb_menu_remove(GtkWidget *window, gpointer data) {
 
   /* disable menu item */
   gtk_widget_set_sensitive(appdata->menu_remove,  FALSE);
+  gtk_widget_set_sensitive(appdata->menu_close,   FALSE);
 
   selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(appdata->gpxview));
   if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
@@ -831,11 +938,15 @@ cb_menu_remove(GtkWidget *window, gpointer data) {
     while(*prev != gpx) prev = &((*prev)->next);
     *prev = gpx->next;
 
+    /* remove gconf entry if file was closed */
+    gconf_remove_closed_name(appdata, gpx->filename);
+
     /* free gpx itself */
     gpx_free(gpx);
 
     /* and remove from store */
     gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
+
   } else {
     g_print ("no row selected.\n");
   }
@@ -1144,6 +1255,7 @@ cb_menu_quit(GtkWidget *window, gpointer data) {
 }
 
 #ifdef USE_MAEMO
+#if MAEMO_VERSION_MAJOR < 5
 static void 
 cb_menu_fullscreen(GtkWidget *widget, gpointer data) {
   appdata_t *appdata = (appdata_t *)data;
@@ -1153,6 +1265,7 @@ cb_menu_fullscreen(GtkWidget *widget, gpointer data) {
   else
     gtk_window_unfullscreen(GTK_WINDOW(appdata->window));
 }
+#endif
 
 static void 
 cb_menu_zoomin(GtkWidget *widget, gpointer data) {
@@ -1242,14 +1355,21 @@ void menu_create(appdata_t *appdata) {
   GtkWidget *submenu = gtk_menu_new();
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
 
-  item = gtk_menu_item_new_with_label( _("Add file...") );
+  item = gtk_menu_item_new_with_label( _("File...") );
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_add), appdata);
 
-  item = gtk_menu_item_new_with_label( _("Add dir...") );
+  item = gtk_menu_item_new_with_label( _("Directory...") );
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_adddir), appdata);
 
+  gtk_menu_append(GTK_MENU_SHELL(submenu), gtk_separator_menu_item_new());
+
+  appdata->menu_close = 
+    item = gtk_menu_item_new_with_label( _("Close") );
+  gtk_menu_append(GTK_MENU_SHELL(submenu), item);
+  g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_close), appdata);
+
   appdata->menu_remove = 
     item = gtk_menu_item_new_with_label( _("Remove") );
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
@@ -1319,11 +1439,13 @@ void menu_create(appdata_t *appdata) {
   submenu = gtk_menu_new();
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
 
+#if MAEMO_VERSION_MAJOR < 5
   appdata->fullscreen_menu_item = 
     item = gtk_check_menu_item_new_with_label( _("Fullscreen") );
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_fullscreen), 
 		   appdata);
+#endif
 
   appdata->zoomout_menu_item = item = gtk_menu_item_new_with_label( _("Zoom +" ));
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
@@ -1638,6 +1760,7 @@ static void crumb_add(appdata_t *appdata, char *name, int level,
     /* disable gpxlist related menu entries */
     gtk_widget_set_sensitive(appdata->menu_import,  FALSE);
     gtk_widget_set_sensitive(appdata->menu_remove,  FALSE);
+    gtk_widget_set_sensitive(appdata->menu_close,   FALSE);
   }
 
   hildon_bread_crumb_trail_push_text(HILDON_BREAD_CRUMB_TRAIL(appdata->bct),
